#---------------------------------------------------------------------------
# Copyright IBM Corp. 2011
# All Rights Reserved
#---------------------------------------------------------------------------
################################################################################
# File name:    system/install/lib/PT/Install/FsCreate.pm
# Description:  Main logic for GFS file system creation and manipulation
# Author:       Joris Mihaeli
################################################################################

################################################################################
# Class name: 	PT::Install::FsCreate
# Description: 	GFS file system creation and manipulation class 
# Base class:     
################################################################################
package PT::Install::FsCreate;

#pragmas
use strict;
use English;
use warnings;

use Switch;
use POSIX;

use PT::Install::BaseDefs;
use PT::Status;
use PT::Logger;
use PT::Install::Screen;
use PT::System::Command;
use PT::System::Common;
use PT::Install::Params::Name;
use PT::Install::ParamsContainer;
use PT::Install::Condition::ListDevices;
use PT::Install::Condition::ListConfigurations;
use PT::Install::Condition::IsFsExtendAllowed;
use PT::Install::Condition::IsDeviceConfigurationComplete;
use PT::Install::ConfigItem::CreatePartition;
use PT::Install::ConfigItem::CreateFileSystem;
use PT::Install::ConfigItem::CreateVolumeGroup;
use PT::Install::ConfigItem::CreatePhysicalVolume;
use PT::Install::ConfigItem::CreateLogicalVolume;
use PT::Install::ConfigItem::RunRemoteCommand;
##
use PT::Install::Strategy::ConfigPartToFs;
use PT::Install::Strategy::ExtendFs;
use PT::Install::Strategy::StopCluster;
use PT::Install::Strategy::StopVtfd;
use PT::Install::Strategy::StartVtfd;
use PT::Install::ConfigItem::StartGfs;
################################################################################
# Name:        new
# Description: constructor
# Params:      void
# Return:      reference to the created instance
################################################################################
sub new
{
   my $class = shift;
   WriteLog(LOG_DEBUG, "new()");
   my $self = bless {}, $class;
   $self->{SCREEN} = PT::Install::Screen->new(); 
   $self->{STATUS} = PT::Status->new();
   $self->{command} = PT::System::Command->new();
   if (CURR_RED_HAT_MAJOR eq RED_HAT_6_MAJOR) {
      $self->{GFS} = 'gfs2';
   }
   else{
      $self->{GFS} = 'gfs';
   }
   return $self;
}
################################################################################
# Name:        init
# Description: initiator
# Params:      
# Return:      status code
################################################################################
sub init
{
   my $self = shift;
   $self->{STATUS}->errno(OK);   # so far
   WriteLog(LOG_DEBUG, "init()");
   $self->{SCREEN}->init();      # if needed
   return $self->{STATUS};
}
################################################################################
# Name:        getAllDevicesConfigStatus
# Description: Determine the configuration status for all devices
# Params:      
# Return:      status code
################################################################################
sub getAllDevicesConfigStatus 
{
   ###########
   # Function steps:
   # 1. Get list of all devices and their sizes
   # 2. Get configuration status for each device
   ###########
   my $self = shift;
   my ($paramContainerRef) = @_;
   WriteLog(LOG_DEBUG, "getAllDevicesConfigStatus()");
   my $status = PT::Status->new();
   $status->errno(OK);   # so far
   my $command = PT::System::Command->new();
   
   # setup Param container
   my $paramCont = PT::Install::ParamsContainer->new();
   
   # Get all devices and their sizes
   my $listCond = PT::Install::Condition::ListDevices->new();
   $listCond->init();
   $status = $listCond->check(\$paramCont);
   if (OK != $status) {
     WriteLog(LOG_ERR, "Unable to get devices list");
     return $status;
   }
         
   my @devsizeList = ();
   my ($entry, $item, $size);
   eval { $size = $paramCont->getParam('size')->getValue();  };
   for (my $index = 1; $index <= $size ; $index++) {
      eval { $item = $paramCont->getParam($index)->getValue();  };
      push (@devsizeList, $item);
   }
  
   # Get all configurations
   my $paramConfigCont = PT::Install::ParamsContainer->new();
   my $listConfigCond = PT::Install::Condition::ListConfigurations->new();
   $listConfigCond->init();
   $status = $listConfigCond->check(\$paramConfigCont);
   if (OK != $status) {
     WriteLog(LOG_ERR, "Unable to get the configurations list");
     return $status;
   }
   
   my @configList = ();
   eval { $size = $paramConfigCont->getParam('size')->getValue();  };
   for (my $index = 1; $index <= $size ; $index++) {
      eval { $item = $paramConfigCont->getParam($index)->getValue();  };
      push (@configList, $item);
   }
     
   # Determine configuration status
   my %seen = ();
   #build lookup table
   foreach $item (@configList) {
      my ($part, $pv_name, $vg_name, $lv_name, $fs_name ) = split (/:/, $item);
      my $device = substr($part, 0, -2);
      if (!defined($fs_name) )  {
         $seen{$device} = ":Partially Configured";
      } else {
         $seen{$device} = ":Configured";
      }
   }
          
   my @deviceStatus = ();  
   for my $entry (@devsizeList) {
      my $tupple = substr($entry,12);
      # check if the device is configured
      my ($device, $size) = split (/:/, $tupple);
      if (exists($seen{$device}) ) {
         $tupple .= $seen{$device};
      } else {
         $tupple .= ":Unused";
      }
      push (@deviceStatus, $tupple);
   }   
      
   # add the size and the accumulated array to the parm container
   my $sizeParam = PT::Install::Params::Name->new(name => 'size');
   $sizeParam->init();
   $sizeParam->setValue(scalar(@deviceStatus) );
   if ( ! $$paramContainerRef->addParam($sizeParam) ) {
       $status->errno(ERROR);
       $status->message('Fail to add parameter');
       return $status;
   }
   my $index = 1;
   for my $dev (@deviceStatus) {
      my $param = PT::Install::Params::Name->new(name => $index);
      $param->init();
      $param->setValue($dev);
      if (! $$paramContainerRef->addParam($param)) {
          $status->errno(ERROR);
          $status->message('Fail to add parameter');
          return $status;
      }
      $index++;
   }
      
   return $status;
}
################################################################################
# Name:        getAllConfigurations
# Description: Determine all device configurations
# Params:      
# Return:      status code
################################################################################
sub getAllConfigurations 
{
   ###########
      # Function steps:
      # 1. Get list of all devices and their sizes
      # 2. Get configuration entities for each device
   ###########
   my $self = shift;
   my ($paramContainerRef) = @_;
   WriteLog(LOG_DEBUG, "getAllConfigurations()");
   my $status = PT::Status->new();
   $status->errno(OK);   # so far
   my $command = PT::System::Command->new();
   
   # setup Param container
   my $paramCont = PT::Install::ParamsContainer->new();
   
   # Get all configurations
   my $listCond = PT::Install::Condition::ListConfigurations->new();
   $listCond->init();
   $status = $listCond->check(\$paramCont);
               
   my @deviceConfig = ();
   my ($size, $item);
   eval { $size = $paramCont->getParam('size')->getValue();  };
   for (my $index = 1; $index <= $size ; $index++) {
      eval { $item = $paramCont->getParam($index)->getValue();  };
      push (@deviceConfig, $item);
   }
         
   # add the configurations to the parm container
   my $sizeParam = PT::Install::Params::Name->new(name => 'size');
   $sizeParam->init();
   $sizeParam->setValue(scalar(@deviceConfig) );
   if ( ! $$paramContainerRef->addParam($sizeParam) ) {
       $status->errno(ERROR);
       $status->message('Fail to add parameter');
       return $status;
   }
   my $index = 1;
   for my $dev (@deviceConfig) {
      my $param = PT::Install::Params::Name->new(name => $index);
      $param->init();
      $param->setValue($dev);
      if (! $$paramContainerRef->addParam($param)) {
          $status->errno(ERROR);
          $status->message('Fail to add parameter');
          return $status;
      }
      $index++;
   }
      
   return $status;
}
################################################################################
# Name:        getAllRepositoryGfsFs
# Description: Get list of all GFS systems that are part of the repository
# Params:      
# Return:      status code
################################################################################
sub getAllRepositoryGfsFs 
{
   ###########
   # Function steps:
   # 1. Collects for all GFS FSs their mount points from /etc/fstab
   # 2. For each mount point checks the existence of mgmt/repository xml files
   # 3. For each repository FS determines its device, size, and mount point
   # 4, Returns the FS data in parameter container
   ###########
   my $self = shift;
   my ($paramContainerRef) = @_;
   my @fsList = ();
   WriteLog(LOG_DEBUG, "getAllRepositoryGfsFs()");
   my $status = PT::Status->new();
   $status->errno(OK);   # so far
 
   #
   my $command = PT::System::Command->new();
   my $cmd = "grep -w $self->{GFS} /etc/fstab 2>/dev/null | awk '{print \$2}'"; 
   $status = $command->run($cmd);   # collect mount points
   if (OK != $status ) {
      WriteLog (LOG_ERR,
        "Failed to get the mount points from /etc/fstab");
      return $status;
   }
   my $mounted_fs = $command->stdout();
   chomp $mounted_fs;
   my @mountList = split(/\n/, $mounted_fs);
   my ($mount, $repository, $management);
   # collect all file systems and their sizes
   my @fsSizeList = ();
   $cmd = "pvs -o pv_name,dev_size --noheadings --separator : --all --units m 2>/dev/null";
   $status = $command->run($cmd);
   if (OK != $status ) {
      WriteLog (LOG_ERR, "Failed to get file systems sizes");
      return $status;
   } else {
      my $output = $command->stdout();
      chomp $output;
      @fsSizeList = split(/\n/, $output);
      for (@fsSizeList) { s/^\s+//};   # remove white space
   }
   for my $mount (@mountList) {
      $repository = "$mount/repository_label.xml";
      $management = "$mount/management_label.xml";
      if (( ! -e $repository) && ( ! -e $management) ) {  
         next;    # skip -> not part of the repository
      } else {
         $status = $self->_get_fs_data($mount, \@fsSizeList, \@fsList);
         if (OK != $status) {
            WriteLog (LOG_ERR, "Failed to get fs for $mount from /etc/fstab");
            return $status;
         }
      }
   }
   # add the accumulated array to the parm container
   my $sizeParam = PT::Install::Params::Name->new(name => 'size');
   $sizeParam->init();
   $sizeParam->setValue(scalar (@fsList));
   if ( ! $$paramContainerRef->addParam($sizeParam) ) {
       $status->errno(ERROR);
       $status->message('Fail to add parameter');
       return $status;
   }
   my $index = 1;
   for my $fs (@fsList) {
      my $param = PT::Install::Params::Name->new(name => $index);
      $param->init();
      $param->setValue($fs);
      if (! $$paramContainerRef->addParam($param)) {
          $status->errno(ERROR);
          $status->message('Fail to add parameter');
          return $status;
      }
      $index++;
   }
      
   return $status;
}
################################################################################
# Name:        getAllNonRepositoryGfsFs
# Description: Get list of all GFS systems that are not part of the repository
# Params:      
# Return:      status code
################################################################################
sub getAllNonRepositoryGfsFs 
{
   ###########
   # Function steps:
   # 1. Collects for all GFS FSs their mount points from /etc/fstab
   # 2. For each mount point checks the existence of mgmt/repository xml files
   # 3. For each non-repository FS determines its device, size, and mount point
   # 4, Returns the FS data in parameter container
   ###########
   my $self = shift;
   my ($paramContainerRef) = @_;
   my @fsList = ();
   WriteLog(LOG_DEBUG, "getAllNonRepositoryGfsFs()");
   my $status = PT::Status->new();
   $status->errno(OK);   # so far
    
   my $command = PT::System::Command->new();
   my $cmd = "grep -w $self->{GFS} /etc/fstab 2>/dev/null | awk '{print \$2}'";
   $status = $command->run($cmd);
   if (OK != $status ) {
      WriteLog (LOG_ERR,
        "Failed to get the mount points from /etc/fstab");
      return $status;
   }
   my $mounted_fs = $command->stdout();
   chomp $mounted_fs;
   my @mountList = split(/\n/, $mounted_fs);
   my ($mount, $repository, $management);
   
   # collect all file systems and their sizes
   my @fsSizeList = ();
   $cmd = "pvs -o pv_name,dev_size --noheadings --separator : --all --units m 2>/dev/null";
   $status = $command->run($cmd);
   if (OK != $status ) {
      WriteLog (LOG_ERR, "Failed to get file systems sizes");
      return $status;
   } else {
      my $output = $command->stdout();
      chomp $output;
      @fsSizeList = split(/\n/, $output);
      for (@fsSizeList) { s/^\s+//};   # remove white space
   }
   
   for my $mount (@mountList) {
      $repository = "$mount/repository_label.xml";
      $management = "$mount/management_label.xml";
      if (( -e $repository) || ( -e $management) ) {  
         next;    # skip -> part of the repository
      } else {
         $status = $self->_get_fs_data($mount, \@fsSizeList, \@fsList);
         if (OK != $status) {
            WriteLog (LOG_ERR, "Failed to get fs for $mount from /etc/fstab");
            return $status;
         }
      }
   }
   # add the accumulated array to the parm container
   my $arraySize = @fsList;
   my $sizeParam = PT::Install::Params::Name->new(name => 'size');
   $sizeParam->init();
   $sizeParam->setValue(scalar (@fsList));
   if ( ! $$paramContainerRef->addParam($sizeParam) ) {
       $status->errno(ERROR);
       $status->message('Fail to add parameter');
       return $status;
   }
   
   my $index = 1;
   for my $fs (@fsList) {
      my $param = PT::Install::Params::Name->new(name => $index);
      $param->init();
      $param->setValue($fs);
      if (! $$paramContainerRef->addParam($param)) {
          $status->errno(ERROR);
          $status->message('Fail to add parameter');
          return $status;
      }
      $index++;
   }
      
   return $status;
}
################################################################################
# Name:        getAllUnconfiguredDevices
# Description: Get list of all unconfigured devices
# Params:      
# Return:      status code
################################################################################
sub getAllUnconfiguredDevices 
{
   ###########
   # Function steps:
   # 1. Collects all devices and their sizes
   # 2. Determines the unused devices
   # 3, Returns the unused devices and their sizes in parameter container
   ###########
   my $self = shift;
   my ($paramContainerRef) = @_;
   WriteLog(LOG_DEBUG, "getAllUnconfiguredDevices()");
   my $status = PT::Status->new();
   my $command = PT::System::Command->new();
   
   # setup Param container
   my $paramCont = PT::Install::ParamsContainer->new();
   
   # Get all devices and their sizes
   my $listCond = PT::Install::Condition::ListDevices->new();
   $listCond->init();
   $status = $listCond->check(\$paramCont);
         
   my @devsizeList = ();
   my ($entry, $item, $device, $size, $vg, $lv, $fs);
   $size=0;
   eval { $size = $paramCont->getParam('size')->getValue();  };
   if ( $size eq "" || ! defined $size) {
      $size=0;
   }
   for (my $index = 1; $index <= $size ; $index++) {
      eval { $item = $paramCont->getParam($index)->getValue();  };
      push (@devsizeList, substr($item,12)); # /dev/mapper/mpath2:1714936.00M
   }  # need to cut the path for later display
     
   # Filter the configured devices  
   my @deviceList = ();
   $status = $self->_filter_configured_devices(\@devsizeList, \@deviceList);
   if (OK != $status) {
      WriteLog(LOG_DEBUG, "Could not filter configured devices");
      return $status;
   }
            
   # add the size and the accumulated array to the parm container
   my $sizeParam = PT::Install::Params::Name->new(name => 'size');
   $sizeParam->init();
   $sizeParam->setValue(scalar(@deviceList) );
   if ( ! $$paramContainerRef->addParam($sizeParam) ) {
       $status->errno(ERROR);
       $status->message('Fail to add parameter');
       return $status;
   }
   my $index = 1;
   for my $dev (@deviceList) {
      my $param = PT::Install::Params::Name->new(name => $index);
      $param->init();
      $param->setValue($dev);
      if (! $$paramContainerRef->addParam($param)) {
          $status->errno(ERROR);
          $status->message('Fail to add parameter');
          return $status;
      }
      $index++;
   }
   
   $status->errno(OK);
   return $status;
}
################################################################################
# Name:        addMissingFstabRecords
# Description: Add FSTAB missing entries
# Params:      
# Return:      status code
################################################################################
sub addMissingFstabRecords 
{
   ###########
   # Function steps:
   # 1. Collect all file system devices from /dev/mapper
   # 2. Finds a subset that is not defined in the /etc/fstab
   # 3. Skip FSs that are not of GFS type
   # 4. Create missing mount points for the remaining FSs
   # 5. Add FS entries in /etc/fstab
   # 6. Mount the newly defined FSs
   # 7. Return list of the added FSs in param container
   ##########
   my $self = shift;
   my ($paramContainerRef) = @_;
   WriteLog(LOG_DEBUG, "addMissingFstabRecords()");
   my $status = PT::Status->new();
   $status->errno(OK);
   my $command = PT::System::Command->new();
   
   # get the devices
   my $cmd = "ls /dev/mapper 2>/dev/null | grep -e '-lv' ";   
   $status = $command->run($cmd);
   my @devsList;
   if (OK == $status) {
      my $devs = $command->stdout();
      chomp $devs;
      @devsList = split(/\n/, $devs);
   } else {
      if ($command->stdout() =~ /^\s*$/) {
         $status->errno(OK);
      } else {
         WriteLog (LOG_ERR, "Command ls failed: ". $command->stderr());
         $status->message("Could not get device list");
      }
      return $status;
   }
   
   my @entries = ();
   for my $dev (@devsList) {
      $cmd = "cat /etc/fstab 2>/dev/null | grep /mnt/$dev";
      $status = $command->run($cmd);
      if (OK != $status) {   # fs not in FSTAB
         # check if the entry is GFS file system
         $cmd = $self->{GFS}."_tool sb /dev/mapper/$dev all 2>/dev/null";
         $cmd .= " | grep 'sb_lockproto' | wc -l";
         $command->run($cmd);
         my $count = $command->stdout();
         chomp $count;
         if ($count == 0) {
            next;    # skip this entry - it is not GFS file system
         }
         
         # add to /etc/fstab
         my $createFS  = PT::Install::ConfigItem::CreateFileSystem->new();
         $status = $createFS->_add_to_fstab($dev);
         if (OK != $status) {
            WriteLog(LOG_ERR, "Failed to add $dev to /etc/fstab");
            return $status;
         }
                           
         # try to mount the added file system
         $status = $createFS->_mount_it($dev);
         if (OK != $status) {
            WriteLog (LOG_ERR, "Failed to mount file system $dev");
            return $status;
         } else {
            WriteLog (LOG_INFO, "File system $dev mounted successfully");
         }
          
         push (@entries, "/dev/mapper/$dev");   # show the added entry         
      }
   }
   $status->errno(OK);  # Set for the display (if we reached till here)
   # add the size and the accumulated array to the parm container
   my $arraySize = @entries;
   my $sizeParam = PT::Install::Params::Name->new(name => 'size');
   $sizeParam->init();
   $sizeParam->setValue($arraySize);
   if ( ! $$paramContainerRef->addParam($sizeParam) ) {
       $status->errno(ERROR);
       $status->message('Fail to add parameter');
       return $status;
   }
      
   my $index = 1;
   for my $entry (@entries) {
      my $param = PT::Install::Params::Name->new(name => $index);
      $param->init();
      $param->setValue($entry);
      if (! $$paramContainerRef->addParam($param)) {
          $status->errno(ERROR);
          $status->message('Fail to add parameter');
          return $status;
      }
      $index++;
   }

   return $status;
}
################################################################################
# Name:        createFsForAllDevices
# Description: Initial creation of file systems on all devices
# Params:      
# Return:      status code
################################################################################
sub createFsForAllDevices 
{
   ###########
   # Function steps:
   # 1. Collects all mpath devices and their sizes
   # 2. Determines whether all the devices are unused
   # 3. Deletes all previously created GFS entries from /etc/fstab
   # 4. Creates file systems on each device
   ###########
   my $self = shift;
   WriteLog(LOG_DEBUG, "createFsForAllDevices()");
   my $status = PT::Status->new();
   $status->errno(OK);

   # check if there are some devices already configured
   $status = $self->_check_devices();
   if (WARN == $status) {
      WriteLog(LOG_ERR, "Configured devices found for CreateFS()!!");
      return $status;
   } elsif (ERROR == $status) {
      WriteLog(LOG_ERR, "Could not determine if there are configured devices");
      return $status;
   }
   
   # Get all devices and their sizes (in MB)
   my @devSizeList = ();
   
   # setup Param container
   my $paramCont = PT::Install::ParamsContainer->new();
   # Get all devices and their sizes
   my $listCond = PT::Install::Condition::ListDevices->new();
   $listCond->init();
   $status = $listCond->check(\$paramCont);
   my ($item, $size);
   eval { $size = $paramCont->getParam('size')->getValue();  };
   for (my $index = 1; $index <= $size ; $index++) {
      eval { $item = $paramCont->getParam($index)->getValue();  };
      push (@devSizeList, substr($item, 12));
   }
   
   # Clean the FSTAB entries
   my $fstabfile = "/etc/fstab";
   my $pattern = "\-lv";
   local $^I = '';
   local @ARGV = ($fstabfile);
   while (<>) {
      next if (m/$pattern/);
      print;
   }
   
   # Create file systems on all devices and set gfs statfst parameter
   $status = $self->_createFileSystems(@devSizeList);
   #PT::System::Common::ptConfigD("restart");
   if (OK != $status) {
      WriteLog(LOG_ERR, "Unable to createFsForAllDevices()");
   } else {
      if (CURR_RED_HAT_MAJOR ne RED_HAT_6_MAJOR) {
         $status = $self->_set_gfs_statfs();
      }
   }
   
   return $status;
}
################################################################################
# Name:        createFsForAllUnconfiguredDevices
# Description: Cretae file systems on all unconfigured devices
# Params:      
# Return:      status code
################################################################################
sub createFsForAllUnconfiguredDevices 
{
   ###########
   # Function steps:
   # 1. Collect all mpath devices and their sizes
   # 2. Discards all configured already devices
   # 3. Creates file systems on each unused device
   ###########
   my $self = shift;
   WriteLog(LOG_DEBUG, "createForAllUnconfiguredDevices()");
   my $status = PT::Status->new();
   $status->errno(OK);
   
   my @fullList = ();
   # setup Param container
   my $paramCont = PT::Install::ParamsContainer->new();
   
   # Get all devices and their sizes
   my $listCond = PT::Install::Condition::ListDevices->new();
   $listCond->init();
   $status = $listCond->check(\$paramCont);
         
   my ($item, $size);
   eval { $size = $paramCont->getParam('size')->getValue();  };
   for (my $index = 1; $index <= $size ; $index++) {
      eval { $item = $paramCont->getParam($index)->getValue();  };
      push (@fullList, substr($item,12));
   }
      
   # Filter the configured devices  
   my @devSizeList = ();
   $status = $self->_filter_configured_devices(\@fullList, \@devSizeList);
   if (OK != $status) {
      WriteLog(LOG_DEBUG, "Could not filter configured devices");
      return $status;
   }
         
   # Create file systems on all unconfigured devices & set gfs statfst parameter
   $status = $self->_createFileSystems(@devSizeList);
   #PT::System::Common::ptConfigD("restart");
   if (OK != $status) {
      WriteLog(LOG_ERR, "Unable to createFsForAllUnconfiguredDevices()");
   } else {
      if (CURR_RED_HAT_MAJOR ne RED_HAT_6_MAJOR) {
         $status = $self->_set_gfs_statfs();
      }
   }
   
   return $status;
}
################################################################################
# Name:        extendFs
# Description: Extend FS to include new device
# Params:      
# Return:      status code
################################################################################
sub extendFs 
{
   ###########
   # Function steps:
   # 1. Retrieve input parms: FS name, unused device, and its size
   # 2. Verify that the future extended FS will be less then 8T in size
   # 3. Create partition and physical volume on the new device
   # 4. Add the PV to the FS volume group
   # 5. Extend the logical volume for this FS with the new storage
   # 6. Extend (grow) the FS
   ###########
   my $self = shift;
   my ($fs_name, $device, $size) = @_;
   WriteLog(LOG_DEBUG, "extendFs()");
   my $status = PT::Status->new();
   $status->errno(OK);
   # Create ParamsContainer
   my $paramCont = PT::Install::ParamsContainer->new();
   $paramCont->init();
   
   # Set up parameters
   my $fsParam = PT::Install::Params::Name->new(name => 'fsname');
   $fsParam->init();
   $fsParam->setValue($fs_name);
   $paramCont->addParam($fsParam);
   
   my $deviceParam =  PT::Install::Params::Name->new(name => 'device');
   $deviceParam->init();
   $deviceParam->setValue($device);
   $paramCont->addParam($deviceParam);
   
   my $sizeParam =  PT::Install::Params::Name->new(name => 'size');
   $sizeParam->init();
   $sizeParam->setValue($size);
   $paramCont->addParam($sizeParam);
         
   # Check if FS extension is possible
   my $extendCond = PT::Install::Condition::IsFsExtendAllowed->new();
   $extendCond->init();
   $status = $extendCond->check(\$paramCont);
   if (OK != $status) {
      WriteLog(LOG_ERR, "failed to get the expand status");
      return $status;
   }
   my $expandFlag = false;
   eval { $expandFlag = $paramCont->getParam('expand')->getValue();  };  
   # Start the action
   if ($expandFlag) {
      my $extendStrategy = PT::Install::Strategy::ExtendFs->new();
      my $status = $extendStrategy->init(params => $paramCont);
      if (OK != $status) {
         WriteLog(LOG_ERR, "failed init extendFs");
         return $status;
      }
      # determine if we are running on a cluster
      my ($numOfNodes, $myNodeId, $nodeAlive);
      ($status, $numOfNodes, $myNodeId) =
                                       PT::System::Common::getCmanNumOfNodes(); 
      if (OK != $status) {
         WriteLog(LOG_WARN, "fail to get cman status");
         return $status;
      }
      #  stop the VTFD service on the local and remote nodes
      WriteLog(LOG_DEBUG, "Stopping the VTFD service on $numOfNodes nodes");
      my $stopVtfd = PT::Install::Strategy::StopVtfd->new();
      WriteLog(LOG_DEBUG, "Starting paramsContainer");
      my $container = PT::Install::ParamsContainer->new();
      my $nameParam = PT::Install::Params::Name->new(name => 'numOfNodes');
      $nameParam->init();
      $nameParam->setValue($numOfNodes);
      $container->addParam($nameParam);
      $stopVtfd->init(params => $container);
      $status = $stopVtfd->run();
      if (OK ne $status) {
         WriteLog(LOG_ERR, "Failed to stop VTFD: ". $status->message());
         return $status;
      }
      # stop remote services
      if ($numOfNodes > 1) {
         my $otherNodeId = $myNodeId ^ 3;
         ($status, $nodeAlive) =
                              PT::System::Common::isCmanNodeAlive($otherNodeId);
         if (OK == $status && $nodeAlive) {
            $status = $self->stopRemoteServices();
            if (OK != $status) {
               WriteLog(LOG_WARN, "fail to stop remote service, clvmd");
               return $status;
            }
         }
      }
      # extend the file system
      $status = $extendStrategy->run();
      if (OK != $status) {
         WriteLog(LOG_ERR, "failed to run extendFs");
      }
      # start the GFS service on the local node
      my $startGfs = PT::Install::ConfigItem::StartGfs->new();
      $status = $startGfs->init();
      if (OK != $status) {
         WriteLog(LOG_ERR, "failed to init GFS service");
         return $status;
      }
      $status = $startGfs->do();
      if (OK != $status) {
         WriteLog(LOG_ERR, "failed to start GFS service");
         return $status;
      }
      # start the VTFD service only on the local node
      my $startVtfd = PT::Install::Strategy::StartVtfd->new();
      my $container1 = PT::Install::ParamsContainer->new();
      $container1->init();
      my $nameParam1 = PT::Install::Params::Name->new(name => 'numOfNodes');
      my $nodes = 1;
      $nameParam1->setValue($nodes);
      $container1->addParam($nameParam1);
      $startVtfd->init(params => $container1);
      $status = $startVtfd->run();
      if (OK ne $status) {
         WriteLog(LOG_WARN, "fail to restart VTFD service");
         return $status;
      }
   } else {
      WriteLog(LOG_ERR, "File System extension would exceed 8TB");
      $status->errno(ERROR);
   }
   return $status;
}
################################################################################
# Name:        partitionDevice
# Description: Create partition on a device
# Params:      
# Return:      status code
################################################################################
sub partitionDevice{
   ###########
   # Function steps:
   # 1. Determine the device configuration status
   # 2. Create partition with the requested size
   ###########
   my $self = shift;
   my ($device, $size) = @_;
   WriteLog(LOG_DEBUG, "partitionDevice($device, $size)");
   my $status = PT::Status->new();
   $status->errno(OK);
      
   # check the configuration for this mpath device
   # drop the check since multiple allowed on a single device
   
   # Create ParamsContainer
   my $paramCont = PT::Install::ParamsContainer->new();
   $paramCont->init();
      
   # Set up parameters
   my $deviceParam =  PT::Install::Params::Name->new(name => 'device');
   $deviceParam->init();
   $deviceParam->setValue($device);
   $paramCont->addParam($deviceParam);
      
   my $sizeParam =  PT::Install::Params::Name->new(name => 'size');
   $sizeParam->init();
   $sizeParam->setValue($size);
   $paramCont->addParam($sizeParam);
                  
   # Create Partition
   my $partitionIt = PT::Install::ConfigItem::CreatePartition->new();
   $status = $partitionIt->init($paramCont);
   if (OK != $status) {
      WriteLog(LOG_ERR, "Failed to initiate partition class");
      return $status;
   } 
   $status = $partitionIt->do();
   if (OK != $status) {
      WriteLog(LOG_ERR, "Failed to create partition on $device");
   }  else  {
      WriteLog(LOG_INFO, "Successfully created partition on $device");
   }
   return $status;
}
###############################################################################
# Name:        deleteFileSystem
# Description: Delete file system from all devices
# Params:      
# Return:      status code
################################################################################
sub  deleteFileSystem 
{
   ###########
   # Function steps:
   # 1. Determine all PVs, VGs, and LVs for the file system to be deleted
   # 2. Remove each one of them in succession
   ###########
   my $self = shift;
   my $fs_name = shift;
   WriteLog(LOG_DEBUG, "deleteFileSystem()");
   my $status = PT::Status->new();
   $status->errno(OK);
   my $command = PT::System::Command->new();
   
   # find out the FS logical entities
   my ($part_name, $pv_name, $vg_name, $lv_name);
   if ($fs_name =~ /(\w+)-(\w+)/ ) {
      $vg_name = $1;
      $lv_name = $2;
   }
   my $cmd = "pvs -o pv_name,vg_name,lv_name --separator : --all --units m";
   $cmd .= " | egrep -v $fs_name | grep $lv_name";
   $status = $command->run($cmd);
   if (OK != $status) {
      WriteLog(LOG_INFO, "Unable to find file systems");
      return $status;
   } else {
      my $out = $command->stdout();
      chomp($out);
      my @entityList = split(/\n/, $out);
      for (@entityList) { s/^\s+// }; # remove white space
      
      $status = $self->_stopServices();
      foreach my $entity (@entityList) {
         ($pv_name, $vg_name, $lv_name) = split /:/, $entity;         
         $pv_name =  substr($pv_name, 12);
         $part_name = $pv_name;
         $status = $self->_erasePartitionConfiguration($part_name, $pv_name,
                                             $vg_name, $lv_name, $fs_name);
      }
   }
   
   return $status;  
}
###############################################################################
# Name:        eraseConfiguration
# Description: Erase configurations on all devices
# Params:      
# Return:      status code
################################################################################
sub  eraseConfiguration {
   ###########
   # Function steps:
   # 1. Collect all devices
   # 2. For each device collect its full configuration 
   # 3. Erase all created entities on each configured device
   ###########
   my $self = shift;
   WriteLog(LOG_DEBUG, "eraseConfiguration()");
   my $status = PT::Status->new();
   $status->errno(OK);
   my $errorFlag = OK;
      
   my $command = PT::System::Command->new();

   # Get all devices and their sizes (in MB)
   my @devSizeList = ();
   
   # setup Param container
   my $paramCont = PT::Install::ParamsContainer->new();
   # Get all devices and their sizes
   my $listCond = PT::Install::Condition::ListDevices->new();
   $listCond->init();
   $status = $listCond->check(\$paramCont);
         
   my ($item, $size);
   eval { $size = $paramCont->getParam('size')->getValue();  };
   for (my $index = 1; $index <= $size ; $index++) {
      eval { $item = $paramCont->getParam($index)->getValue();  };
      push (@devSizeList, substr($item, 12));
   }
   
   # select configured devices
   $status = $self->_stopServices();
   my ($device, $entry);
   foreach $entry (@devSizeList) {
      ($device, $size) = split /:/, $entry;
      # get the configuration for this mpath device
      my $configCont = PT::Install::ParamsContainer->new();
      my $devParam = PT::Install::Params::Name->new(name => 'devname');
      $devParam->init();
      $devParam->setValue($device);
      $configCont->addParam($devParam);
      my $condition = PT::Install::Condition::IsDeviceConfigurationComplete->new();
      $condition->init();
      my $configStatus = 0;
      $status = $condition->check(\$configCont);
      if (OK == $status) {
         eval { $configStatus = $configCont->getParam('config')->getValue();  };
      } else {
         WriteLog(LOG_INFO, "Could not determine device configuration");
         $errorFlag = ERROR;
         next;    # skip this device
      }
      if ($configStatus != 0) {
         $status = $self->_eraseDeviceConfiguration($configCont, $device);      
         if (OK != $status) {
	     $errorFlag = ERROR;         
   	   }
      }
   }
      
   if ($errorFlag == ERROR ) {
       WriteLog(LOG_ERR, "Unable to eraseConfiguration On All Devices");
       $status->errno(ERROR);
   }
      
   return $status;  
}
###############################################################################
# Name:        eraseDeviceConfiguration
# Description: Erase configurations on particular device
# Params:      
# Return:      status code
################################################################################
sub  eraseDeviceConfiguration {
   ###########
   # Function steps:
   # 1. Determine the full configuration of the device to be erased
   # 2. Erase all created entities
   ###########
   my $self = shift;
   my $device = shift;
   WriteLog(LOG_DEBUG, "eraseDeviceConfiguration()");
   my $status = PT::Status->new();
   $status->errno(OK);
   my $errorFlag = OK;
   my $command = PT::System::Command->new();
   
   # get the configuration for this mpath device
   my $configCont = PT::Install::ParamsContainer->new();
   my $devParam = PT::Install::Params::Name->new(name => 'devname');
   $devParam->init();
   $devParam->setValue($device);
   $configCont->addParam($devParam);
   my $condition = PT::Install::Condition::IsDeviceConfigurationComplete->new();
   $condition->init();
   my $configStatus = 0;
   $status = $condition->check(\$configCont);
   if (OK == $status) {
      eval { $configStatus = $configCont->getParam('config')->getValue();  };
   } else {
      WriteLog(LOG_INFO, "Could not determine device configuration");
      $status->errno(ERROR);
      return $status; 
   }
   if ($configStatus != 0) {
      $status = $self->_stopServices();
      $status = $self->_eraseDeviceConfiguration($configCont, $device);
   } else {
      WriteLog(LOG_INFO, "The device to be erased is not configured");
   }
   
   return $status;  
}
###############################################################################
# Name:        cleanFileSystemsSpace
# Description: Remove all partial configurations
# Params:      
# Return:      status code
################################################################################
sub  cleanFileSystemsSpace 
{
   ###########
   # Function steps:
   # 1. Determine all partial configurations
   # 2. Erase the partial configurations
   ###########
   my $self = shift;
   my $status = PT::Status->new();
   $status->errno(OK);
   # setup Param container
   my $paramCont = PT::Install::ParamsContainer->new();
   
   # Get all configurations
   my $listCond = PT::Install::Condition::ListConfigurations->new();
   $listCond->init();
   $status = $listCond->check(\$paramCont);
   if (OK != $status) {
      WriteLog(LOG_ERR, "Could not determine device configuration");
      return $status;
   }
   my @deviceConfig = ();
   my ($size, $item);
   eval { $size = $paramCont->getParam('size')->getValue();  };
   for (my $index = 1; $index <= $size ; $index++) {
      eval { $item = $paramCont->getParam($index)->getValue();  };
      push (@deviceConfig, $item);
   }
   for my $tupple (@deviceConfig) {
      my ($part, $pv_name, $vg_name, $lv_name, $fs_name ) =
                                                split (/:/, $tupple);
      if (defined($fs_name) && $fs_name ne "") {   # fs exists
         next;
      } else {
         $fs_name = "";
      }
      if (!defined($lv_name) ) {
         $lv_name = "";
      }
      if (!defined($vg_name) ) {
         $vg_name = "";
      }
      if (!defined($pv_name) ) {
         $pv_name = "";
      }
      # remove partial configurations
      $status = $self->_erasePartitionConfiguration($part, $pv_name, $vg_name,
                                                   $lv_name, $fs_name);
      if (OK != $status) {
         WriteLog(LOG_ERR, "Failed to remove partial configuration $tupple");
         last;
      } else {
         WriteLog(LOG_DEBUG, "Removed partial configuration $tupple");
      }
   }
   
   return $status;
}   
###############################################################################
# Name:        removePartialConfiguration
# Description: Remove partail configuration
# Params:      Configuration tupple
# Return:      status code
################################################################################
sub  removePartialConfiguration {
   ###########
   # Function steps:
   # 1. Collect the partially created entities
   # 2. Erase the created entities
   ###########
   my $self = shift;
   my $configuration = shift;
   WriteLog(LOG_DEBUG, "removePartialConfiguration()");
   my $status = PT::Status->new();
   $status->errno(OK);
   
   my ($part, $pv_name, $vg_name, $lv_name, $fs_name ) =
                                                split (/:/, $configuration);
   if (defined($fs_name) && $fs_name ne "") {
      WriteLog(LOG_INFO, "Good configuration was passed for removal");
      $status->errno(ERROR);
      return $status;   # this is good configuration - why it was passed?
   }
   if (!defined($lv_name) ) {
      $lv_name = "";
   }
   if (!defined($vg_name) ) {
      $vg_name = "";
   }
   if (!defined($pv_name) ) {
      $pv_name = "";
   }
   if (!defined($part) || $part eq "") {
      $status->errno(ERROR);
      return $status;   # something is wrong here
   }
   # shouldnt be need to stop services because no FS exists
   $status = $self->_erasePartitionConfiguration($part, $pv_name, $vg_name,
                                                 $lv_name, $fs_name);
   return $status;
}
###############################################################################
# Name:        scanStorageInterconnects
# Description: Scan Storage Interconnects
# Params:      
# Return:      status code
################################################################################
sub scanStorageInterconnects
{
   my $self = shift;
   WriteLog(LOG_DEBUG, "START");
   my $status = PT::Status->new();
   $status->errno(OK);
   WriteLog (LOG_DEBUG, "Scanning FC host adapters");
   
   # Get list of FC host adapters
   my @hostList = ();
   my $command = PT::System::Command->new();
   my $cmd = "ls -1 /sys/class/fc_host 2>/dev/null";
   $status = $command->run($cmd);
   if (OK != $status ) {
      WriteLog (LOG_ERR, "Failed to get list of fibre channel host adapters");
      return $status;
   } else {    
      my $output = $command->stdout();
      chomp ($output);
      @hostList = split(/\n/, $output);
   }
 
   # Rescan the storage interconnects
   for my $host (@hostList) {
      if ( -w ("/sys/class/scsi_host/".$host."/scan") ) {
         WriteLog (LOG_DEBUG, "Start scanning for $host");
         my $file = "/sys/class/scsi_host/".$host."/scan";
         open my $FILE, ">", $file;
         print $FILE "- - -";
         close $FILE;
         $status = $self->_udev_settle(); # no need to check the status
      } else {
         WriteLog (LOG_DEBUG, "Missing 'scan' capability for $host");
      }
      sleep(10);  # wait a bit for the dust to settle down
      $cmd = "multipath -v 2";
      $status = $command->run($cmd);   
      sleep(10);  # wait a bit for the dust to settle down
   }
   
   # Rebuild the LVM cache to reflect possible device changes
   WriteLog (LOG_DEBUG, "Rebuild the LVM cache");
   my $clustername = PT::System::Common::getLiveClusterName();
   if ($clustername) {
      # cluster is running
      $cmd = "vgscan >/dev/null";
      $status = $command->run($cmd);
      if (OK != $status ) {
         WriteLog (LOG_ERR, "Failed to run vgscan to rebuild the LVM cache");
      }
   }
   $status->errno(OK);
   return $status;
}

sub _udev_settle
{
   my $self = shift;
   my $status = PT::Status->new();
   $status->errno(OK);
   my $command = PT::System::Command->new();
   my $cmd;
   if ( -x "/sbin/udevadm" ) {
      $cmd = "/sbin/udevadm settle 2>/dev/null";
      $status = $command->run($cmd);   
      if (OK != $status ) {
         WriteLog (LOG_ERR, "Failed to run udevadm");
      }
   } elsif ( -x "/sbin/udevsettle") {
      $cmd = "/sbin/udevsettle 2>/dev/null";
      $status = $command->run($cmd);   
      if (OK != $status ) {
         WriteLog (LOG_ERR, "Failed to run udevsettle");
      }
   } else {
      sleep(3);   # probably enough
   }
   
   return $status;
}

sub _createFileSystems
{
   ###########
   # Function steps:
   # 1. Determine the cluster name
   # 2. Determine for each device the number of partitions to be created
   #    and their sizes
   # 3. Create partition(s)
   # 4. Create file system(s)
   ###########
   my $self = shift;
   my (@devsizeList) = @_;
   WriteLog(LOG_DEBUG, "start");
   my $status = PT::Status->new();
   $status->errno(OK);
   #PT::System::Common::ptConfigD("stop");
   # determine the cluster name
   my $clusterName = PT::System::Common::getLiveClusterName();  
   if ( ! $clusterName) {
      $status->errno(ERROR);
      $status->message("Fail to get cluster name");
      return $status;
   }
   my ($dev_name, $size, $sizeX, $sizeY);
   my @partitionList = ();
   for my $entry (@devsizeList) {
      ($dev_name, $size) = split (/:/, $entry);
      chop($size);    # remove M char
      if ($size > SIZE8T) { # divide into min number of equal partitions
         my $num_partitions = ceil($size / SIZE8T);   # number of partitions
         if ($num_partitions > 4 ) {
            WriteLog(LOG_INFO, "Device $dev_name size $size not supported");
            $status->errno(ERROR);
            return $status;    # stop further processing
         }
         $sizeX = floor($size / $num_partitions);    # size of a partition
         for( my $index = 1; $index <= $num_partitions; $index++) {
            if ($index == $num_partitions) {    # use full remaining size
               $sizeY = $self->_get_device_free_space($dev_name);
               if ($sizeY ne "") {
                  $sizeX = $sizeY;
               }
            }
            # Create partition
            $status = $self->partitionDevice($dev_name, $sizeX);
            my $partition = $dev_name."p".$index;
            if (OK != $status) { 
               foreach my $item (@partitionList) {
                  $self->_erasePartitionConfiguration($item,"","","","");
               }
               return $status;    # stop further processing 
            } else {
               push (@partitionList, $partition);    ##
            }
         }
      } else {
         # create partition
         $status = $self->partitionDevice($dev_name, $size);
         my $partition = $dev_name."p1";
         if (OK != $status) { 
            foreach my $item (@partitionList) {
               $self->_erasePartitionConfiguration($item,"","","","");
            }
            return $status;    # stop further processing 
         } else {
            push (@partitionList, $partition);    ##
         }
      }
   }
   # Execute kpartx on all devices on the other node
   my $cmd = "/opt/dtc/app/sbin/FSManager -k";
   $status = $self->_run_remote($cmd);
   if (OK != $status) {
      WriteLog(LOG_ERR, "Failed to create device maps remotely");
      return $status;
   }
   # Build file systems
   foreach my $partition (@partitionList) {
      $status = $self->_createFs($partition, $clusterName);
      if (OK != $status) {
         $self->_erasePartitionConfiguration($partition,"","","","");
         return $status;    # stop further processing 
      }
   }
   # Add remotely to /etc/fstab and mount file systems
   $cmd = "/opt/dtc/app/sbin/FSManager -T";
   $status = $self->_run_remote($cmd);
   if (OK != $status) {
      WriteLog(LOG_ERR, "Failed to add remotely to /etc/fstab");
      return $status;
   }
  
   return $status;
}

sub _createFs
{
   ###########
   # Function steps:
   # 1. Prepare initial parameters; partition, cluser
   # 1. Create in succession PV, VG, LV, and file system
   ###########
   my $self = shift;
   my ($partition, $cluster) = @_;
   my $status = PT::Status->new();
   $status->errno(OK);
   
   # Create unique VG, LV, FS identifier
   my $uid = $self->_createUniqueID();
   
   # Create ParamsContainer
   my $paramCont = PT::Install::ParamsContainer->new();
   $paramCont->init();
   
   # add parameters
   my $part_param = PT::Install::Params::Name->new(name => 'partition');
   $part_param->init();
   $part_param->setValue($partition);
   $paramCont->addParam($part_param);
   
   my $clustParam = PT::Install::Params::Name->new(name => 'cluster');
   $clustParam->init();
   $clustParam->setValue($cluster);
   $paramCont->addParam($clustParam);
   
   my $identParam = PT::Install::Params::Name->new(name => 'identifier');
   $identParam->init();
   $identParam->setValue($uid);
   $paramCont->addParam($identParam);
   
   # run strategy
   my $strategy = PT::Install::Strategy::ConfigPartToFs->new();
   $status = $strategy->init(params => $paramCont);
   if (OK != $status) {
      WriteLog(LOG_ERR, "failed init create FS strategy");
      return $status;
   }
   $status = $strategy->run();
   if (OK != $status) {
      WriteLog(LOG_ERR, "failed to run create FS strategy");
   }
   
   return $status;
}

sub _run_remote
{
   my $self = shift;
   my $cmd = shift;
   my $status = PT::Status->new();
   $status->errno(OK);
   
   # Check if we are on a cluster
   my ($numOfNodes, $myNodeId, $nodeAlive);
   ($status, $numOfNodes, $myNodeId) = PT::System::Common::getCmanNumOfNodes(); 
   if (OK != $status) {
      WriteLog(LOG_WARN, "Failed to get cman status - continuing locally");
      $status->errno(OK);
      return $status;
   }
   if ($numOfNodes > 1) {
      my $otherNodeId = $myNodeId ^ 3;
      ($status, $nodeAlive) = PT::System::Common::isCmanNodeAlive($otherNodeId);
      if (OK == $status && $nodeAlive) {
         my $remoteCmd = PT::Install::ConfigItem::RunRemoteCommand->new();      
         my $cmdObj = PT::Install::Params::Name->new(name => 'commandToRun');
         $cmdObj->init();
         $cmdObj->setValue($cmd);
         my $otherPc = PT::Install::ParamsContainer->new();
         $otherPc->init();
         $otherPc->addParam($cmdObj);
         $status = $remoteCmd->init($otherPc);
         if (OK != $status) {
            WriteLog (LOG_ERR, "Failed to init remote cmd $cmd");
            return $status;
         }
         $status = $remoteCmd->do();
         if (OK != $status) {
            WriteLog (LOG_ERR, "Failed to run remote cmd $cmd");
            return $status;
         }
      } else {
         WriteLog(LOG_WARN, "Remote node is down - continuing locally");
         $status->errno(OK);
      }
   }
   
   return $status;
}

sub _createUniqueID
{
   my $self = shift;
   WriteLog(LOG_DEBUG, "_createUniqueID()");
   my $unique_string;
   my $status = PT::Status->new();
   $status->errno(OK);
   my $command = PT::System::Command->new();
   # Get file systems list
   my @fsList = ();
   my $cmd = "ls /dev/mapper | grep vgfs | sort";
   $command->run($cmd); # we get list of entries like: vgfs0001-lvfs0001  
   if (OK != $status ) {
      WriteLog (LOG_ERR, "Failed to get file systems list");
      return $status;
   } else {
      my $output = $command->stdout();
      chomp $output;
      @fsList = split(/\n/, $output);
      for (@fsList) { s/^\s+//};   # remove white space
   }

   my $fsNumber = scalar (@fsList);
   my $number;
   if ($fsNumber == 0) {
      $number = $fsNumber; 
   } else {
      my $entry = $fsList[$fsNumber-1];   # last file system in the list
      if ( $entry =~ m/^vgfs(\d+)\S+$/) {
         $number = $1;
      }
   }
   $unique_string = $number + 1; # assign next number
   my $length = IDENT_LNGTH;
   my $format = "%." . $length . "d";  # 4 digits format
   $unique_string = sprintf($format, $unique_string);
   
   return $unique_string;
}

sub _stopServices
{
   my $self = shift;
   WriteLog(LOG_DEBUG, "_stopServices()");
   my $status = PT::Status->new();
   $status->errno(OK);
   my $command = PT::System::Command->new();
   $status = $command->run("service vtfd stop");
   $status = $command->run("service gfs stop");
   # hopefully no need to stop clvmd !?!
   
   return $status;
}

sub _eraseDeviceConfiguration
{
   ###########
   # Function steps:
   # 1. Erase partition configuration
   # 2. Nullify the tables on the device
   ###########
   my $self = shift;
   my ($paramContainer, $device) = @_;
   WriteLog(LOG_DEBUG, "_eraseDeviceConfiguration()");
   my $status = PT::Status->new();
   $status->errno(OK);
   my $errorFlag = OK;

   my $size;
   eval { $size = $paramContainer->getParam('size')->getValue();  };
   my ($item, $part_name, $pv_name, $vg_name, $lv_name, $fs_name);
   for (my $index = 1;$index <= $size ; $index++) {
      eval { $item = $paramContainer->getParam($index)->getValue();  };
      ($part_name, $pv_name, $vg_name, $lv_name, $fs_name) = split (/:/, $item);
      $status = $self->_erasePartitionConfiguration($part_name, $pv_name,
      			                      $vg_name, $lv_name, $fs_name);
      if (OK != $status) {
      	 $errorFlag = ERROR;         
      }
   }
   if ($errorFlag == ERROR ) {
      WriteLog(LOG_ERR, "Failed to erase some entities on $device");
      $status->errno(ERROR);
   } else {
      # Zeroing data - total annihilation
      $device = "/dev/mapper/$device";
      WriteLog(LOG_INFO, "Zeroing data on $device");
      my $cmd = "dd if=/dev/zero of=$device bs=1024k count=12";
      my $command = PT::System::Command->new();
      $status = $command->run($cmd); 
   }
   
   return $status;   

}

sub _erasePartitionConfiguration
{
   ###########
   # Function steps:
   # 1. Unmount the file system and remove its entry from /etc/fstab
   # 2. Deactivate and remove LV
   # 3. Deactivate and remove VG
   # 4. Remove PV
   # 5. Remove partition
   my $self = shift;
   my ($part_name, $pv_name, $vg_name, $lv_name, $fs_name) = @_;
   WriteLog(LOG_DEBUG, "_erasePartitionConfiguration()");
      
   my $status = PT::Status->new();
   $status->errno(OK);
    
   my $command = PT::System::Command->new();
   my $cmd;
   if (defined($fs_name) && $fs_name ne "") {
      $cmd = "umount /mnt/$fs_name";
      $status = $command->run($cmd);
      # remove from FSTAB
      my $fstabfile = "/etc/fstab";
      local $^I = '';
      local @ARGV = ($fstabfile);
      while (<>) {
         next if (m/$fs_name/);
         print;
      }
   }
   if ( defined($vg_name) && $vg_name ne "" &&
        defined($lv_name) && $lv_name ne "" ) {
      $cmd = "lvchange -a n /dev/$vg_name/$lv_name";
      $status = $command->run($cmd);
      $cmd = "lvremove -f /dev/$vg_name/$lv_name";
      $status = $command->run($cmd);
   }
   if (defined($vg_name) && $vg_name ne "") {
      $cmd = "vgchange -a n $vg_name";
      $status = $command->run($cmd);
      $cmd = "vgremove -f $vg_name";
      $status = $command->run($cmd);
   }
   if (defined($pv_name) && $pv_name ne "") {
      $cmd = "pvremove -ffy /dev/mapper/$pv_name";
      $status = $command->run($cmd);
   }
   if (defined($part_name) && $part_name ne "") {
      #
      # construct device & partition number from the partition name 
      #
      # example - assuming that $part_name = "mpath10p2"
      my $mp = "/dev/mapper/".substr($part_name, 0, -2); # device is: "mpath10"
      my $pnum = substr($part_name, -1); # partition number is: "2"
      $cmd = "parted -s $mp rm $pnum";   # "parted -s /dev/mapper/mpath10 rm 2"
      $status = $command->run($cmd);
   }

   return $status;
}

sub _get_device_free_space
{
   my $self = shift;
   my $device = shift;
   my $status = PT::Status->new();
   $status->errno(OK);
   my $cmd = "parted -s /dev/mapper/$device unit \'MiB\' print free | tail -1";
   $cmd .= " | grep Free | awk '{print \$3}'";
   my $command = PT::System::Command->new();
   $status = $command->run($cmd);
   my $size;
   if (OK != $status) {
      $size = "";
   } else {
      $size = $command->stdout();
      chomp($size);
      if (substr($size,-3) eq "MiB") {
         chop($size);
         chop($size);
         chop($size);
      } else {
         $size = "";
      }   
   }
   return $size;   
}

sub _set_gfs_statfs
{
   my $self = shift;
   my $status = PT::Status->new();
   $status->errno(OK);   
   my $command = PT::System::Command->new();
   # invoke set_gfs_statfs_fast for background parallel operation
   my $cmd = "/opt/dtc/app/sbin/set_gfs_statfs_fast";
   $status = $command->run($cmd);  
   
   return $status;
}

sub _check_devices
{
   my $self = shift;
   my $status = PT::Status->new();
   $status->errno(OK);
   my $command = PT::System::Command->new();
   $status = $command->run("ls /dev/mapper/ | grep mpath");
   if (OK != $status) {    # impossible sitiuation
      WriteLog(LOG_DEBUG, "Failed list command ??");
      return $status;
   } else {
      my $output = $command->stdout();
      if ($output =~ /mpath\d+p/) { # found configured device
         $status->errno(WARN);
      }
   }
    
   return $status;
}

sub _filter_configured_devices
{
   my $self = shift;
   my ($fullListRef, $deviceListRef) = @_;
   my $status = PT::Status->new();
   $status->errno(OK);
    
   $status = $self->_check_devices();  # check if there are configured devices
   if (OK == $status) {                # No configured devices found
      foreach my $entry (@$fullListRef) { 
         push(@$deviceListRef, $entry);
      }
   } elsif ( ERROR == $status) {
      return $status;
   } else {    # configured devices found
      # Filter the configured devices
      my $command = PT::System::Command->new();
      $status = $command->run("ls /dev/mapper/ | grep mpath");
      my @devList = (); 
      if (OK != $status) {
         WriteLog(LOG_DEBUG, "Failed to check if device is configurered");
         return $status;
      } else {
         my $output = $command->stdout();
         chomp $output;
         @devList = split(/\n/, $output);
      }   
      foreach my $entry (@$fullListRef) {
         my ($device, $size) = split /:/, $entry;
         my $filter = $device . "p";
         my @match = grep /$filter/, @devList;
         if (0 == scalar(@match)) {
             push (@$deviceListRef, $entry);
         }    
      }
      $status->errno(OK);
   }
    
   return $status;
}

sub _get_fs_data
{
   my $self = shift;
   my ($mount, $fsSizeListRef, $fsListRef) = @_;
   my $status = PT::Status->new();
   $status->errno(OK);   
   my $command = PT::System::Command->new();
   my $cmd = "grep $mount /etc/fstab | awk '{print \$1}'";
   $status = $command->run($cmd);
   if (OK != $status ) {
      WriteLog (LOG_ERR, "Failed to get fs for $mount from /etc/fstab");
      return $status;
   } else {    # get the file system & its size
      my $fs = $command->stdout();
      chomp ($fs);
      my $tupple = "";
      foreach my $entry (@$fsSizeListRef) {
         if ($entry =~ /$fs/ ) {
            $tupple = $entry . ":" . $mount;
            last;
         }
      }
      if ($tupple eq "") {
         $tupple = "$fs:unknown:$mount";
      }
      push (@$fsListRef, $tupple);
   }
   
   return $status;
}

sub stopRemoteServices
{
   my $self = shift;
   WriteLog(LOG_DEBUG, "START");
   my $status = PT::Status->new();
   my $remoteStop = PT::Install::Strategy::StopCluster->new();
   my $paramCont = PT::Install::ParamsContainer->new();
   my $paramNodeId = PT::Install::Params::Name->new(name => 'nodeToStop');
   my $paramNumNodes = PT::Install::Params::Name->new(name => 'numOfNodes');
   $paramNumNodes->setValue(2);
   $paramNodeId->setValue(2);
   $paramCont->addParam($paramNodeId);
   $paramCont->addParam($paramNumNodes);
   $status = $remoteStop->init(params => $paramCont);
   if (OK != $status) {
      WriteLog(LOG_WARN, "fail to stop remote services");
      return $status;
   }
   $status = $remoteStop->run();
   if (OK != $status) {
      WriteLog(LOG_WARN, "fail to stop remote services");
      return $status;
   }
   sleep (60);
   #allow 60 seconds for the system to stabilize.
   return $status;
}

1;


